###
# Copyright (c) 2011, Rebecca Bettencourt
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

###

import popen2
import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

class Code(callbacks.Plugin):
    """This plugin contains commands for evaluating expressions in various scripting languages."""
    threaded = True

    def py(self, irc, msg, args, text):
        """<text>

        Prints the result of the given Python expression.
        """
        cmd = self.registryValue('python.command')
        if cmd:
            if 'import' in text:
            	irc.error('Cannot use import here!')
            	return
            args = [cmd, '-c', 'print ' + text]
            inst = popen2.Popen4(args)
            (r, w) = (inst.fromchild, inst.tochild)
            try:
                lines = r.readlines()
                lines = map(str.rstrip, lines)
                lines = filter(None, lines)
                if lines:
                	for line in lines:
                		irc.reply(line, prefixNick=False)
                else:
                    irc.error('It seems the configured Python interpreter was '
                              'not available.')
            finally:
                w.close()
                r.close()
                inst.wait()
        else:
            irc.error('I couldn\'t find the Python interpreter on this system. '
                      'If it is installed on this system, reconfigure the '
                      'supybot.plugins.Code.python.command configuration '
                      'variable appropriately.')
    py = wrap(py, ['text'])

    def pl(self, irc, msg, args, text):
        """<text>

        Prints the result of the given Perl expression.
        """
        cmd = self.registryValue('perl.command')
        if cmd:
            if 'import' in text:
            	irc.error('Cannot use import here!')
            	return
            if 'use' in text:
            	irc.error('Cannot use use here!')
            	return
            if 'require' in text:
            	irc.error('Cannot use require here!')
            	return
            args = [cmd, '-e', 'print ' + text + ';']
            inst = popen2.Popen4(args)
            (r, w) = (inst.fromchild, inst.tochild)
            try:
                lines = r.readlines()
                lines = map(str.rstrip, lines)
                lines = filter(None, lines)
                if lines:
                	for line in lines:
                		irc.reply(line, prefixNick=False)
                else:
                    irc.error('It seems the configured Perl interpreter was '
                              'not available.')
            finally:
                w.close()
                r.close()
                inst.wait()
        else:
            irc.error('I couldn\'t find the Perl interpreter on this system. '
                      'If it is installed on this system, reconfigure the '
                      'supybot.plugins.Code.perl.command configuration '
                      'variable appropriately.')
    pl = wrap(pl, ['text'])

    def php(self, irc, msg, args, text):
        """<text>

        Prints the result of the given PHP expression.
        """
        cmd = self.registryValue('php.command')
        if cmd:
            if 'include' in text:
            	irc.error('Cannot use include here!')
            	return
            if 'require' in text:
            	irc.error('Cannot use require here!')
            	return
            args = [cmd, '-r', 'print ' + text + ';']
            inst = popen2.Popen4(args)
            (r, w) = (inst.fromchild, inst.tochild)
            try:
                lines = r.readlines()
                lines = map(str.rstrip, lines)
                lines = filter(None, lines)
                if lines:
                	for line in lines:
                		irc.reply(line, prefixNick=False)
                else:
                    irc.error('It seems the configured PHP interpreter was '
                              'not available.')
            finally:
                w.close()
                r.close()
                inst.wait()
        else:
            irc.error('I couldn\'t find the PHP interpreter on this system. '
                      'If it is installed on this system, reconfigure the '
                      'supybot.plugins.Code.php.command configuration '
                      'variable appropriately.')
    php = wrap(php, ['text'])

    def rb(self, irc, msg, args, text):
        """<text>

        Prints the result of the given Ruby expression.
        """
        cmd = self.registryValue('ruby.command')
        if cmd:
            if 'require' in text:
            	irc.error('Cannot use require here!')
            	return
            args = [cmd, '-e', 'print ' + text]
            inst = popen2.Popen4(args)
            (r, w) = (inst.fromchild, inst.tochild)
            try:
                lines = r.readlines()
                lines = map(str.rstrip, lines)
                lines = filter(None, lines)
                if lines:
                	for line in lines:
                		irc.reply(line, prefixNick=False)
                else:
                    irc.error('It seems the configured Ruby interpreter was '
                              'not available.')
            finally:
                w.close()
                r.close()
                inst.wait()
        else:
            irc.error('I couldn\'t find the Ruby interpreter on this system. '
                      'If it is installed on this system, reconfigure the '
                      'supybot.plugins.Code.ruby.command configuration '
                      'variable appropriately.')
    rb = wrap(rb, ['text'])

    def xn(self, irc, msg, args, text):
        """<text>

        Prints the result of the given XION expression.
        """
        cmd = self.registryValue('xion.command')
        if cmd:
            args = [cmd, '-s', 'deny', '-e', text]
            inst = popen2.Popen4(args)
            (r, w) = (inst.fromchild, inst.tochild)
            try:
                lines = r.readlines()
                lines = map(str.rstrip, lines)
                lines = filter(None, lines)
                if lines:
                	for line in lines:
                		irc.reply(line, prefixNick=False)
                else:
                    irc.error('It seems the configured XION interpreter was '
                              'not available.')
            finally:
                w.close()
                r.close()
                inst.wait()
        else:
            irc.error('I couldn\'t find the XION interpreter on this system. '
                      'If it is installed on this system, reconfigure the '
                      'supybot.plugins.Code.xion.command configuration '
                      'variable appropriately.')
    xn = wrap(xn, ['text'])

Class = Code

# vim:set shiftwidth=4 softtabstop=4 expandtab textwidth=79:
